﻿Imports ThinkGeo.MapSuite.Layers
Imports System.Collections.ObjectModel

Public Class ZoomLevelHelper
  Public Shared ZoomLevels As Collection(Of CustomZoomLevel)
  Public Enum EnumSymbolType
    PointSymbol = 1
    LineSymbol = 2
    AreaSymbol = 3
    None = 4
  End Enum
  Public Enum EnumDrawingBehavior
    DrawLayer = 1
    DrawLabelsAndSelection = 2
    DrawEverything = 3
    Unknown = 4
  End Enum
  Public Enum EnumScaleMode
    ShowAtAllScales = 1
    ShowAtSetScales = 2
    UseLayerScaleRanges = 3
  End Enum
  Public Enum EnumScaleRangeSymbolBehavoir
    UseSameSymbol = 1
    UseSeparateSymbols = 2
  End Enum
  Public Enum EnumZoomSpecsType
    DrawingStyle = 1
    LabelStyle = 2
  End Enum
  Public Shared ReadOnly Property MaxZoomLevel As CustomZoomLevel
    Get
      Return New CustomZoomLevel(129761280 * 2)
    End Get
  End Property
  Public Shared ReadOnly Property MinZoomLevel As CustomZoomLevel
    Get
      Return New CustomZoomLevel(0)
    End Get
  End Property
  Public Shared Function GetNextHigherZoomLevel(ByVal oCurrentZoom As CustomZoomLevel) As CustomZoomLevel
    Dim oZoom As CustomZoomLevel
    Dim oNextZoom As CustomZoomLevel = Nothing

    For i = ZoomLevels.Count - 1 To 0 Step -1
      oZoom = ZoomLevels(i)
      If oZoom.Scale > oCurrentZoom.Scale Then
        oNextZoom = oZoom
        Exit For
      End If
    Next i

    '***** If we could not find the next higher zoom level it means 
    '***** oCurrentZoom is already the highest zoom
    If oNextZoom Is Nothing Then
      oNextZoom = New CustomZoomLevel(-1)
    End If

    Return oNextZoom
  End Function
  Public Shared Function GetNextHigherScale(ByVal dScale As Double) As Double
    Dim oZoom As CustomZoomLevel
    Dim oNextZoom As CustomZoomLevel = Nothing

    For i = ZoomLevels.Count - 1 To 0 Step -1
      oZoom = ZoomLevels(i)
      If oZoom.Scale > dScale Then
        oNextZoom = oZoom
        Exit For
      End If
    Next i

    '***** If we could not find the next higher zoom level it means 
    '***** oCurrentZoom is already the highest zoom
    If oNextZoom Is Nothing Then
      oNextZoom = New CustomZoomLevel(-1)
    End If

    Return oNextZoom.Scale
  End Function

  Public Shared Sub InitZoomLevels()
    '***** Initialize default set of zoom levels
    Dim oZoomLevel As CustomZoomLevel

    ZoomLevels = New Collection(Of CustomZoomLevel)
    oZoomLevel = New CustomZoomLevel(129761280) '***** 1" = 2048 miles, 129,761,280
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(64880640) '***** 1" = 1024 miles
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(32440320) '***** 1" = 512 miles
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(24330240) '***** 1" = 384 miles
    ZoomLevels.Add(oZoomLevel)



    oZoomLevel = New CustomZoomLevel(16220160) '***** 1" = 256 miles
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(8110080) '***** 1" = 128 miles
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(4055040) '***** 1" = 64 miles
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(3041280) '***** 1" = 48 miles
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(2027520) '***** 1" = 32 miles
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(1520640) '***** 1" = 24 miles
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(1013760) '***** 1" = 16 miles
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(760320) '***** 1" = 12 miles
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(506880) '***** 1" = 8 miles
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(443520) '***** 1" = 7 miles
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(380160) '***** 1" = 6 miles
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(316800) '***** 1" = 5 miles
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(253440) '***** 1" = 4 miles
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(221760)  '***** 1" = 3 1/2 mile
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(190080)  '***** 1" = 3 mile
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(158400)  '***** 1" = 2 1/2 mile
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(126720) '***** 1" = 2 miles
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(110880)  '***** 1" = 1 3/4 mile
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(95040)  '***** 1" = 1 1/2 mile
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(79200)  '***** 1" = 1 1/4 mile
    ZoomLevels.Add(oZoomLevel)

    'oZoomLevel = New CustomZoomLevel(84480)  '***** 1" = 1 1/3 mile
    'ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(63360)  '***** 1" = 1 mile
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(56320)  '***** 1" = 0.8888 mile
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(50688)  '***** 1" = 0.8 mile
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(42240)  '***** 1" = 02/3 mile
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(36206)  '***** 1" = 0.5714 mile
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(31680)  '***** 1" = 0.5 mile
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(28160)  '***** 1" = 0.4444 mile
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(25344)  '***** 1" = 0.4 mile
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(24000)  '***** 1" = 2000 feet
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(21120)  '***** 1" = 1760 feet
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(18000)  '***** 1" = 1500 feet
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(16800)  '***** 1" = 1400.00 feet
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(15840)  '***** 1" = 1320 feet
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(14080)  '***** 1" = 1173.33 feet
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(12000)  '***** 1" = 1000 feet
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(10800)  '***** 1" = 900 feet
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(9000)   '***** 1" = 750 feet
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(7920)   '***** 1" = 660 feet
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(6000)   '***** 1" = 500 feet
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(4800)   '***** 1" = 400 feet
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(4200)   '***** 1" = 350 feet
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(3600)   '***** 1" = 300 feet
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(3000)   '***** 1" = 250 feet
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(2400)   '***** 1" = 200 feet
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(1800)   '***** 1" = 150 feet
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(1200)   '***** 1" = 100 feet
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(900)   '***** 1" = 75 feet
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(600)   '***** 1" = 50 feet
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(450)   '***** 1" = 37.5 feet
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(300)   '***** 1" = 25 feet
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(150)   '***** 1" = 12.5 feet
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(75)   '***** 1" = 6.25 feet
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(12)   '***** 1" = 1 foot
    ZoomLevels.Add(oZoomLevel)

    oZoomLevel = New CustomZoomLevel(1)   '***** 1" = 1"
    ZoomLevels.Add(oZoomLevel)

  End Sub
  Public Shared Sub ApplyCustomZoomLevels(oZoomLevelSet As ZoomLevelSet)

    With oZoomLevelSet

      .CustomZoomLevels.Clear()
      For Each oZoomLevel As CustomZoomLevel In ZoomLevels
        .CustomZoomLevels.Add(oZoomLevel)
      Next oZoomLevel
    End With
  End Sub

  ''' <summary>
  ''' Loads the combo box on the tool bar that sets the Map scale
  ''' </summary>
  ''' <param name="oComboBoxTool"></param>
  Public Shared Sub LoadScaleComboZoomLevels(oComboBoxTool As ComboBox)
    oComboBoxTool.DataSource = Nothing
    For Each oZoomLevel In ZoomLevels
      oZoomLevel.Text = "1:" + CInt(oZoomLevel.Scale).ToString
    Next oZoomLevel
    oComboBoxTool.DataSource = ZoomLevels
    oComboBoxTool.ValueMember = "Scale"
    oComboBoxTool.DisplayMember = "Text"
  End Sub
  Public Shared Function SetListByValueMember(ByVal oListControl As ComboBox, ByVal oValue As Object) As Boolean
    Dim cboComboBox As System.Windows.Forms.ComboBox
    Dim bSuccess As Boolean = False
    Dim i As Integer = 0

    cboComboBox = CType(oListControl, ComboBox)
    cboComboBox.SelectedIndex = -1
    For Each oCustomZoomLevel In cboComboBox.Items
      If ObjectsHaveEqualValues(oCustomZoomLevel.scale, oValue) Then
        cboComboBox.SelectedIndex = i
        bSuccess = True
        Exit For
      End If
      i += 1
    Next oCustomZoomLevel
    '***** The value was found and the selected item set
    Return bSuccess
  End Function
  Public Shared Function ObjectsHaveEqualValues(ByVal oObject1 As Object, ByVal oObject2 As Object) As Boolean
    Dim dDouble1 As Double
    Dim dDouble2 As Double
    Dim nLong1 As Long
    Dim nLong2 As Long

    'QB #948, Jamie Irwin 9/18/2012, We have a problem if either oObject1 or 2 is a GUID, so
    'we altred the If Block to catch that case.
    If (TypeOf oObject1 Is Guid Or TypeOf oObject2 Is Guid) And
         (Not TypeOf oObject1 Is Guid Or Not TypeOf oObject2 Is Guid) Then
      Return False
    ElseIf TypeOf oObject1 Is Single Or
         TypeOf oObject1 Is Double Then
      dDouble1 = CType(oObject1, Double)
      dDouble2 = CType(oObject2, Double)
      Return dDouble1 = dDouble2
    ElseIf TypeOf oObject1 Is Integer Or
             TypeOf oObject1 Is Long Or
             TypeOf oObject1 Is Short Or
             TypeOf oObject1 Is [Enum] Then
      nLong1 = CType(oObject1, Long)
      nLong2 = CType(oObject2, Long)
      Return nLong1 = nLong2
    Else
      Return oObject1.Equals(oObject2)
    End If
  End Function


End Class